//------------------------------------------------------------------
// Stochastic Momentum Index developed by William Blau
//------------------------------------------------------------------
#property copyright "www.forex-station.com"
#property link      "www.forex-station.com"

#property indicator_separate_window
#property indicator_buffers 6
#property indicator_color1  clrLimeGreen
#property indicator_color2  clrLimeGreen
#property indicator_color3  clrPaleVioletRed
#property indicator_color4  clrPaleVioletRed
#property indicator_color5  clrDarkGray
#property indicator_color6  clrGold
#property indicator_width1  2
#property indicator_width3  2
#property indicator_width5  2
#property indicator_width6  2
#property indicator_level1  0
#property indicator_levelcolor clrDarkGray
#property strict


//
//
//
//
//

enum enMaTypes
{
   ma_sma,    // Simple moving average
   ma_ema,    // Exponential moving average
   ma_smma,   // Smoothed MA
   ma_lwma    // Linear weighted MA
};

extern int       SMIPeriod1   = 8;                // Smi period 1
extern int       SMIPeriod2   = 15;               // Smi period 2
extern int       SMIPeriod3   = 2;                // Smi period 3
extern int       SignalPeriod = 5;                // Signal period
extern enMaTypes MaMethod     = ma_lwma;          // Average type

//
//
//
//
//


double smi[];
double smihuu[];
double smihud[];
double smihdd[];
double smihdu[];
double sig[];
double slope[];

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

int init()
{
   IndicatorBuffers(7);
   SetIndexBuffer(0,smihuu); SetIndexStyle(0, DRAW_HISTOGRAM);
   SetIndexBuffer(1,smihud); SetIndexStyle(1, DRAW_HISTOGRAM);
   SetIndexBuffer(2,smihdd); SetIndexStyle(2, DRAW_HISTOGRAM);
   SetIndexBuffer(3,smihdu); SetIndexStyle(3, DRAW_HISTOGRAM);
   SetIndexBuffer(4,smi);
   SetIndexBuffer(5,sig);
   SetIndexBuffer(6,slope);
   IndicatorShortName("Blau Smi ("+(string)SMIPeriod1+","+(string)SMIPeriod2+","+(string)SMIPeriod3+","+(string)SignalPeriod+")");
   return(0);
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

int start()
{
   int counted_bars=IndicatorCounted();
      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
         int limit = fmin(Bars-counted_bars,Bars-1);

   //
   //
   //
   //
   //

      double alfa = 2.0 / (1.0 + SignalPeriod);
      for (int i=limit; i>=0; i--)
      {
         if (i<Bars-1) 
         {
           double highest = High[iHighest(NULL,0,MODE_HIGH,SMIPeriod1,i)];
           double lowest  = Low [iLowest (NULL,0,MODE_LOW ,SMIPeriod1,i)];
           double sm      = Close[i]-(highest+lowest)*0.5;
           double hl      =           highest-lowest;
           double value1  = iCustomMa(MaMethod,iCustomMa(MaMethod,sm,SMIPeriod2,i,0),SMIPeriod3,i,1);
           double value2  = iCustomMa(MaMethod,iCustomMa(MaMethod,hl,SMIPeriod2,i,2),SMIPeriod3,i,3);
            
            if (value2!=0)
                  smi[i] = 100.00*(value1/(0.5*value2));
            else  smi[i] = 0;
                  sig[i] = sig[i+1] + alfa*(smi[i]-sig[i+1]);
                  
                  //
                  //
                  //
                  //
                  //
                  
                  smihuu[i] = EMPTY_VALUE;
                  smihud[i] = EMPTY_VALUE;
                  smihdd[i] = EMPTY_VALUE;
                  smihdu[i] = EMPTY_VALUE;
                  slope[i] = (i<Bars-1) ? (smi[i]>smi[i+1]) ? 1 : (smi[i]<smi[i+1]) ? -1 : slope[i+1] : 0;
                  if (smi[i]>0)
                  if (slope[i]==1)
                        smihuu[i] = smi[i];
                  else  smihud[i] = smi[i];
                  if (smi[i]<0)
                  if (slope[i]==1)
                        smihdu[i] = smi[i];
                  else  smihdd[i] = smi[i];
          }
      }
return(0);
}

//------------------------------------------------------------------
//                                                                  
//------------------------------------------------------------------
//
//
//
//
//

#define _maInstances 4
#define _maWorkBufferx1 1*_maInstances
#define _maWorkBufferx2 2*_maInstances

string averageName(int mode)
{
   switch (mode)
   {
      case ma_sma   : return("SMA");
      case ma_ema   : return("EMA");
      case ma_smma  : return("SMMA");
      case ma_lwma  : return("LWMA");
   }
   return("");
}
double iCustomMa(int mode, double price, double length, int r, int instanceNo=0)
{
   r = Bars-r-1;
   switch (mode)
   {
      case ma_sma   : return(iSma(price,(int)length,r,instanceNo));
      case ma_ema   : return(iEma(price,length,r,instanceNo));
      case ma_smma  : return(iSmma(price,length,r,instanceNo));
      case ma_lwma  : return(iLwma(price,(int)length,r,instanceNo));
      default       : return(price);
   }
}

//
//
//
//
//

double workSma[][_maWorkBufferx2];
double iSma(double price, int period, int r, int instanceNo=0)
{
   if (period<=1) return(price);
   if (ArrayRange(workSma,0)!= Bars) ArrayResize(workSma,Bars); instanceNo *= 2; int k;

   //
   //
   //
   //
   //
      
   workSma[r][instanceNo+0] = price;
   workSma[r][instanceNo+1] = price; for(k=1; k<period && (r-k)>=0; k++) workSma[r][instanceNo+1] += workSma[r-k][instanceNo+0];  
   workSma[r][instanceNo+1] /= 1.0*k;
   return(workSma[r][instanceNo+1]);
}

//
//
//
//
//

double workEma[][_maWorkBufferx1];
double iEma(double price, double period, int r, int instanceNo=0)
{
   if (ArrayRange(workEma,0)!= Bars) ArrayResize(workEma,Bars);

   //
   //
   //
   //
   //
      
   workEma[r][instanceNo] = price;
   if (r>0 && period>1)
          workEma[r][instanceNo] = workEma[r-1][instanceNo]+(2.0 / (1.0+period))*(price-workEma[r-1][instanceNo]);
   return(workEma[r][instanceNo]);
}

//
//
//
//
//

double workSmma[][_maWorkBufferx1];
double iSmma(double price, double period, int r, int instanceNo=0)
{
   if (period<=1) return(price);
   if (ArrayRange(workSmma,0)!= Bars) ArrayResize(workSmma,Bars);

   //
   //
   //
   //
   //

   workSmma[r][instanceNo] = price;
   if (r>0 && period>1)
          workSmma[r][instanceNo] = workSmma[r-1][instanceNo]+(price-workSmma[r-1][instanceNo])/period;
   return(workSmma[r][instanceNo]);
}

//
//
//
//
//

double workLwma[][_maWorkBufferx1];
double iLwma(double price, double period, int r, int instanceNo=0)
{
   if (ArrayRange(workLwma,0)!= Bars) ArrayResize(workLwma,Bars);
   
   //
   //
   //
   //
   //
   
   period = fmax(period,1);
   workLwma[r][instanceNo] = price;
      double sumw = period;
      double sum  = period*price;

      for(int k=1; k<period && (r-k)>=0; k++)
      {
         double weight = period-k;
                sumw  += weight;
                sum   += weight*workLwma[r-k][instanceNo];  
      }             
      return(sum/sumw);
}
